require 'rexml/document'
require 'facter'

module PuppetX
  module EnterpriseModules
    module OraInstall
      module Facts

        def get_su_command
          os = Facter.value(:kernel)
          if os == 'Linux'
            return 'su -l '
          elsif os == 'SunOS'
            return 'su - '
          end
          'su -l '
        end

        def get_ora_inv_path
          os = Facter.value(:kernel)
          if 'Linux' == os
            return '/etc'
          elsif 'SunOS' == os
            return '/var/opt/oracle'
          end
          '/etc'
        end

        def get_opatch_version(home)
          version = lsinventory(home).scan(/OPatch version    : (.*)/).flatten.first
          Puppet.debug "ora_install opatch version #{home}/OPatch/opatch: #{version}"
          (version.nil?) ? 'Invalid' : version
        end

        def get_orainst_loc
          if File.exist?(get_ora_inv_path + '/oraInst.loc')
            str = ''
            output = File.read(get_ora_inv_path + '/oraInst.loc')
            output.split(/\r?\n/).each do |item|
              if item.match(/^inventory_loc/)
                str = item[14, 50]
              end
            end
            return str
          else
            return ''
          end
        end

        def lsinventory(home)
          @lsinventory ||= {}
          @lsinventory[home] ||= Facter::Util::Resolution.exec("su #{user_for_file(home)} -c 'ORACLE_HOME=#{home} #{home}/OPatch/opatch lsinventory'")
        end

        def get_homes(path)
          unless path.nil?
            if File.exist?(path + '/ContentsXML/inventory.xml')
              file = File.read(path + '/ContentsXML/inventory.xml')
              doc = REXML::Document.new(file)
              homes = []
              doc.elements.each('/INVENTORY/HOME_LIST/HOME') do |element|
                str = element.attributes['LOC']
                unless str.nil?
                  if str.include? 'plugins'
                    # skip EM agent
                  elsif str.include? 'jdk'
                    # skip EM agent
                  elsif str.include? 'OraPlaceHolderDummyHome'
                    # skip EM agent
                  else
                    homes << str
                  end
                end
              end
              return homes
            else
              return {}
            end
          else
            return {}
          end
        end

        def get_version(home)
          version = opatch_installed?(home) ? lsinventory(home).scan(/^(?:Oracle Database|Oracle Client|Oracle Grid Infrastructure|Oracle GoldenGate)\s.+?\s+(.*)$/).flatten.first : 'Invalid'
          (version.nil?) ? 'Invalid' : version
        end

        def get_installed_patches(oracle_home)
          if File.directory?(oracle_home)
            ora_user = user_for_file(oracle_home)
            patches_in_home(oracle_home, ora_user).flatten.sort
          else
            ['Invalid']
          end
        end

        def patches_in_home(oracle_product_home_dir, os_user)
          if opatch_installed?(oracle_product_home_dir)
            lsinventory(oracle_product_home_dir).scan(/Patch\s.(\d+)\s.*:\sapplied on/).flatten
          else
            #
            # When Opatch is not installed, we don't report any patches
            #
            ['Invalid']
          end
        end

        def opatch_installed?(oracle_home)
          File.exist?("#{oracle_home}/OPatch/opatch")
        end

        def user_for_file(home)
          #
          # To run opatch, we need the owner of the grid and oracle directory. The mechanism
          # we use to get this user, is to fetch the owner of the bin directory in the grid or
          # oracle home. Sometimes however, these directories also contain files with root as owner.
          # since root can never be the user of the grid or oracle processen, we skip those.
          #
          a_file = first_non_root_file(home)
          uid = File.stat("#{home}/#{a_file}").uid
          Etc.getpwuid(uid).name
        end

        def first_non_root_file(dir)
          Dir.new(dir).each { |file| break file if File.stat("#{dir}/#{file}").uid.nonzero? }
        end

        def cleanup_opatch_residue(homes)
          # Some older versions of Opatch leave files and directories arround
          # When running opatch often, this causes some issues. This is to make
          # sure all files created by running Opatch as part of the fact, are remopved
          basename = Time.now.strftime("%Y-%m-%d_%H-%M-*%p")
          Puppet.debug `rm -rf /tmp/OraInstall*`
          homes.each {|home| FileUtils.rm_f("#{home}/cfgtoollogs/opatch/lsinv/lsinventory#{basename}.txt") }
        end

        def homes_hash
          Puppet.debug('getting version information directly')
          homes = get_homes(get_orainst_loc)
          homes_array = homes.collect do |home|
            [home, { 'product_version'   => get_version(home),
                     'opatch_version'    => get_opatch_version(home),
                     'installed_patches' => get_installed_patches(home) }]
          end
          cleanup_opatch_residue(homes) if Facter.value(:kernel) == 'Linux'
          Hash[homes_array]
        end
      end
    end
  end
end

