#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# ora_install::installdb
#
# @summary You can use this class to install a working oracle database on your system.
# This defined type supports the following versions of Oracle:
# 
# - 11.2.0.1
# - 11.2.0.3
# - 11.2.0.4
# - 12.1.0.1
# - 12.1.0.2
# - 12.2.0.1
# - 18.0.0.0
# 
# ## Example
# 
# Here is an example on how you can use this class to install an Oracle database
# on your system.
# 
# ```puppet
# ora_install::installdb{ '12.1.0.2_Linux-x86-64':
#   version                   => '12.1.0.2',
#   file                      => 'V46095-01',
#   database_type             => 'SE',
#   oracle_base               => '/oracle',
#   oracle_home               => '/oracle/product/12.1/db',
#   bash_profile              => true,
#   user                      => 'oracle',
#   group                     => 'dba',
#   group_install             => 'oinstall',
#   group_oper                => 'oper',
#   download_dir              => '/data/install',
#   zip_extract               => true,
#   puppet_download_mnt_point => $puppet_download_mnt_point,
# }
# ```
#
# @param [Enum['SE2', 'SE', 'EE', 'SEONE']] database_type
#    Selects the type of database you want to install.
#    At this point in time the following database types are supported and allowed:
#    - `EE`     : Enterprise Edition   (all versions)
#    - `SE`     : Standard Edition     (versions <= 12.1.0.1)
#    - `SEONE`  : Standard Edition One (versions <= 12.1.0.1)
#    - `SE2`    : Standard Edition Two (versions >= 12.1.0.2)
#    The default value is: `SE2`
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Boolean] bash_profile
#    Create a bash profile for the specified user or not.
#    Valid values are `true` and `false`.
#    When you specify a `true` for the parameter, Puppet will create a standard bash profile for the specified user. The bash profile will be placed in a directory named `user_base_dir/user`.
#    ```puppet
#    ora_install::client { 'Oracle client':
#      ...
#      bash_profile  => true,
#      user          => 'oracle',
#      user_base_dir => '/home',
#      ...
#    }
#    ```
#    This snippet will create a bash profile called `/home/oracle/.bash_profile`.
#
# @param [Boolean] cleanup_install_files
#    Cleanup extracted files after use.
#    This is a boolean value. When you set this value to `true`. The installer class will
#    remove all extracted zip files after it has done its work.
#    The default value is: `true`
#    Here is an example:
#    ```puppet
#    ora_install::....{...
#      ...
#      cleanup_install_files => false,  # Keep all unzipped files
#      ...
#    }
#    ```
#
# @param [Optional[String[1]]] cluster_nodes
#    A comma seperated list of the nodes running in the RAC cluster.
#    Example: `node1,node2`
#
# @param [Optional[String[1]]] ee_optional_components
#    This variable is used to enable or disable custom install.
#    When its is set to  true, the attribute `ee_options_selection` is used. When this value is false, the attribute `ee_options_selection` is ignored.
#
# @param [Boolean] ee_options_selection
#    List of Enterprise Edition Options you would like to install.
#    Check the oracle documentation what values are valid.
#
# @param [String[1]] file
#    The source file to use.
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [String[1]] group_install
#    The os group to use for installation.
#    The default value is: `oinstall`
#
# @param [String[1]] group_oper
#    The OS group to allow operator rights.
#    The default value is: `oper`
#
# @param [Boolean] is_rack_one_install
#    This variable is used to enable or disable RAC One Node install.
#    - true  : Value of RAC One Node service name is used.
#    - false : Value of RAC One Node service name is not used.
#    The default value is: `false`
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#    The directory that contains the oracle inventory.
#    The default value is: `/oracle_base/oraInventory`
#
# @param [Stdlib::Absolutepath] oracle_base
#    A directory to use as Oracle base directory.
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [Optional[Boolean]] remote_file
#    **DEPRECATED**
#    This parameter is not being used anymore and will be removed in a future release.
#    `true`.
#
# @param [Stdlib::Absolutepath] temp_dir
#    Directory to use for temporary files.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [Stdlib::Absolutepath] user_base_dir
#    The directory to use as base directory for the users.
#
# @param [Enum['18.0.0.0',
#     '12.2.0.1',
#     '12.1.0.1',
#     '12.1.0.2',
#     '11.2.0.3',
#     '11.2.0.4',
#     '11.2.0.1']] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Boolean] zip_extract
#    The specified source file is a zip file that needs te be extracted.
#    When you specify a value of false, the source attribute must contain a reference to a directory instead of a zip file.
#    The default value is: `true`
#
#--++--
define ora_install::installdb(
  # lint:ignore:2sp_soft_tabs
  Enum['18.0.0.0','12.2.0.1','12.1.0.1','12.1.0.2','11.2.0.3','11.2.0.4', '11.2.0.1']
                                               $version                   = undef,
  Stdlib::Absolutepath                         $oracle_base               = undef,
  Stdlib::Absolutepath                         $oracle_home               = undef,
  String[1]                                    $file                      = undef,
  Enum['SE2', 'SE', 'EE', 'SEONE']             $database_type             = 'SE2',
  Optional[Stdlib::Absolutepath]               $ora_inventory_dir         = undef,
  Boolean                                      $ee_options_selection      = false,
  Optional[String[1]]                          $ee_optional_components    = undef, # 'oracle.rdbms.partitioning:11.2.0.4.0,oracle.oraolap:11.2.0.4.0,oracle.rdbms.dm:11.2.0.4.0,oracle.rdbms.dv:11.2.0.4.0,oracle.rdbms.lbac:11.2.0.4.0,oracle.rdbms.rat:11.2.0.4.0'
  Boolean                                      $bash_profile              = true,
  String[1]                                    $user                      = 'oracle',
  Stdlib::Absolutepath                         $user_base_dir             = '/home',
  String[1]                                    $group                     = 'dba',
  String[1]                                    $group_install             = 'oinstall',
  String[1]                                    $group_oper                = 'oper',
  Stdlib::Absolutepath                         $download_dir              = '/install',
  Stdlib::Absolutepath                         $temp_dir                  = '/tmp',
  Boolean                                      $zip_extract               = true,
  Optional[String[1]]                          $puppet_download_mnt_point = undef,
  Optional[Boolean]                            $remote_file               = undef,
  Optional[String[1]]                          $cluster_nodes             = undef,
  Boolean                                      $cleanup_install_files     = true,
  Boolean                                      $is_rack_one_install       = false,
  Variant[Boolean, Enum['on_failure']]         $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
  # lint:endignore
)
{
  ora_install { $title: ensure => 'present'}

  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  unless ( $::kernel in ['Linux','SunOS'] ) {
    fail('Unrecognized operating system, please use it on a Linux or SunOS host')
  }

  unless ( $oracle_base in $oracle_home ){
    fail('oracle_home folder should be under the oracle_base folder')
  }

  case $version {
    '11.2.0.1', '12.1.0.1': {
      unless ( $database_type in ['EE', 'SE', 'SEONE'] ) {
        fail("Unrecognized database_type(${database_type}) use EE, SE or SEONE for version ${version}")
      }
      $files = ["${file}_1of2.zip", "${file}_2of2.zip"]
      $extract_staged = true
      $install_command = "/bin/sh -c 'unset DISPLAY;${download_dir}/${file}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/db_install_${version}.rsp'"
      $install_creates = "${oracle_home}/dbs"
    }
    '11.2.0.3', '11.2.0.4': {
      unless $database_type in ['EE', 'SE', 'SEONE'] {
        fail("Unrecognized database_type(${database_type}) use EE, SE or SEONE for version ${version}")
      }
      $files =  ["${file}_1of7.zip", "${file}_2of7.zip"]
      $extract_staged = true
      $install_command = "/bin/sh -c 'unset DISPLAY;${download_dir}/${file}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/db_install_${version}.rsp'"
      $install_creates = "${oracle_home}/dbs"
      }
    '12.1.0.2': {
      unless $database_type in ['EE', 'SE'] {
        fail("Unrecognized database_type(${database_type}) use EE or SE2 for version ${version}")
      }
      $files = ["${file}_1of2.zip", "${file}_2of2.zip"]
      $extract_staged = true
      $install_command = "/bin/sh -c 'unset DISPLAY;${download_dir}/${file}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/db_install_${version}.rsp'"
      $install_creates = "${oracle_home}/dbs"
    }
    '12.2.0.1': {
      unless $database_type in ['EE', 'SE2'] {
        fail("Unrecognized database_type(${database_type}) use EE or SE2 for version ${version}")
      }
      $files = ["${file}.zip"]
      $extract_staged = true
      $install_command = "/bin/sh -c 'unset DISPLAY;${download_dir}/${file}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/db_install_${version}.rsp'"
      $install_creates = "${oracle_home}/dbs"
    }
    '18.0.0.0': {
      unless $database_type in ['EE', 'SE2'] {
        fail("Unrecognized database_type(${database_type}) use EE or SE2 for version ${version}")
      }
      $files = ["${file}.zip"]
      $extract_staged = false
      $install_command = "/bin/sh -c 'unset DISPLAY;${oracle_home}/runInstaller -silent -waitforcompletion -ignorePrereq -responseFile ${download_dir}/db_install_${version}.rsp'"
      $install_creates = "${oracle_home}/oraInst.loc"
    }
    default: {}
  }

  $exec_path     = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  if ( $puppet_download_mnt_point ) {
    $mount_point     = $puppet_download_mnt_point
  } else {
    $mount_point     = 'puppet:///modules/ora_install/'
  }

  if ( $mount_point == $download_dir ) {
    $archive_cleanup = false
  } else {
    $archive_cleanup = true
  }

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${oracle_base}/oraInventory"
  }

  unless ( oracle_exists( $oracle_home ) ) {

    db_directory_structure{ "oracle structure ${version}":
      ensure            => present,
      oracle_base_dir   => $oracle_base,
      ora_inventory_dir => $ora_inventory,
      download_dir      => $download_dir,
      os_user           => $user,
      os_group          => $group_install,
    }

    if ( $zip_extract ) {

      if ( $extract_staged ) {
        file { "${download_dir}/${file}":
          ensure => directory,
          owner  => $user,
          group  => $group,
          mode   => '0755',
        }

        $files.each | $filename | {
          archive { "${download_dir}/${filename}":
            ensure       => present,
            before       => Exec["install oracle database ${title}"],
            cleanup      => $archive_cleanup,
            creates      => "${download_dir}/${file}/installdb",
            group        => $group_install,
            extract      => true,
            extract_path => "${download_dir}/${file}",
            require      => Db_directory_structure["oracle structure ${version}"],
            source       => "${mount_point}/${filename}",
            temp_dir     => $temp_dir,
            user         => $user,
          }
        }
      } else {
        file { $oracle_home:
          ensure  => directory,
          recurse => false,
          replace => false,
          mode    => '0775',
          owner   => $user,
          group   => $group_install,
          require => Db_directory_structure["oracle structure ${version}"],
        }

        $files.each | $filename | {
          archive { "${download_dir}/${filename}":
            ensure       => present,
            before       => Exec["install oracle database ${title}"],
            cleanup      => $archive_cleanup,
            creates      => "${oracle_home}/installdb",
            extract      => true,
            extract_path => $oracle_home,
            group        => $group_install,
            require      => File[$oracle_home],
            source       => "${mount_point}/${filename}",
            temp_dir     => $temp_dir,
            user         => $user,
          }
        }
      }
    }

    ora_install::utils::dborainst{"database orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group_install,
    }

    unless ( defined(File["${download_dir}/db_install_${version}.rsp"]) ) {
      file { "${download_dir}/db_install_${version}.rsp":
        ensure  => present,
        content => template("ora_install/db_install_${version}.rsp.erb"),
        mode    => '0775',
        owner   => $user,
        group   => $group,
        require => [Ora_install::Utils::Dborainst["database orainst ${version}"],
                    Db_directory_structure["oracle structure ${version}"]],
      }
    }

    exec { "install oracle database ${title}":
      command     => $install_command,
      creates     => $install_creates,
      environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
      timeout     => 0,
      returns     => [6,0],
      path        => $exec_path,
      user        => $user,
      group       => $group_install,
      cwd         => $oracle_base,
      logoutput   => $logoutput,
      require     => [Ora_install::Utils::Dborainst["database orainst ${version}"],
                      File["${download_dir}/db_install_${version}.rsp"]],
    }

    if ( $bash_profile ) {
      unless ( defined(File["${user_base_dir}/${user}/.bash_profile"]) ) {
        file { "${user_base_dir}/${user}/.bash_profile":
          ensure  => present,
          content => regsubst(template('ora_install/bash_profile.erb'), '\r\n', "\n", 'EMG'),
          mode    => '0775',
          owner   => $user,
          group   => $group,
        }
      }
    }

    exec { "run root.sh script ${title}":
      command   => "${oracle_home}/root.sh",
      user      => 'root',
      group     => 'root',
      path      => $exec_path,
      cwd       => $oracle_base,
      logoutput => $logoutput,
      require   => Exec["install oracle database ${title}"],
    }

    if ( $extract_staged ) {
      file { $oracle_home:
        ensure  => directory,
        recurse => false,
        replace => false,
        mode    => '0775',
        owner   => $user,
        group   => $group_install,
        require => Exec["install oracle database ${title}","run root.sh script ${title}"],
      }
    }

    # cleanup
    if ( $cleanup_install_files ) {
      if ( $zip_extract ) {
        exec { "remove oracle db extract folder ${title}":
          command   => "rm -rf ${download_dir}/${file}",
          user      => 'root',
          group     => 'root',
          path      => $exec_path,
          cwd       => $oracle_base,
          logoutput => $logoutput,
          require   => [Exec["install oracle database ${title}"],
                        Exec["run root.sh script ${title}"],],
        }
      }
      if ( $archive_cleanup ) {
        $files.each | $filename | {
          file { "${download_dir}/${filename}":
            ensure => absent,
          }
        }
      }
    }
  }
}
