#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# ora_install::net
#
# @summary Installs and configures Oracle SQL*Net
# This defined type allows you to install and configure Oracle SQL*net.
# 
# ```puppet
# ora_install::net{ 'config net8':
#   oracle_home   => '/oracle/product/11.2/db',
#   version       => '11.2',
#   user          => 'oracle',
#   group         => 'dba',
#   download_dir  => '/install',
#   db_port       => 1521,
# }
# ```
#
# @param [Integer] db_port
#    The IP port number to use for connecting to the database.
#    The default value is: `1521`
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [Stdlib::Absolutepath] temp_dir
#    Directory to use for temporary files.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [Enum['11.2', '12.1', '12.2', '18.0']] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
#--++--
define ora_install::net(
  Stdlib::Absolutepath                $oracle_home  = undef,
  Enum['11.2','12.1','12.2','18.0']   $version      = '11.2',
  String[1]                           $user         = 'oracle',
  String[1]                           $group        = 'dba',
  Integer                             $db_port      = 1521,
  Stdlib::Absolutepath                $download_dir = '/install',
  Stdlib::Absolutepath                $temp_dir     = '/tmp',
  Variant[Boolean,Enum['on_failure']] $logoutput    = lookup({name => 'logoutput', default_value => 'on_failure'}),
){
  ora_install { $title: ensure => 'present'}

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  file { "${download_dir}/netca_${version}.rsp":
    ensure  => present,
    content => template("ora_install/netca_${version}.rsp.erb"),
    mode    => '0775',
    owner   => $user,
    group   => $group,
  }

  exec { "install oracle net ${title}":
    command     => "${oracle_home}/bin/netca -silent -responsefile ${download_dir}/netca_${version}.rsp",
    cwd         => $temp_dir,
    require     => File["${download_dir}/netca_${version}.rsp"],
    creates     => "${oracle_home}/network/admin/listener.ora",
    path        => $exec_path,
    user        => $user,
    group       => $group,
    environment => ["USER=${user}",],
    logoutput   => $logoutput,
  }
}
