#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# ora_install::opatchupgrade
#
# @summary Installs or upgrades the specified version of the Oracle OPatch utility.
# For `opatchupgrade` you need to provide the Oracle support csi_number and supportId and need to be online. Or leave them empty but it needs the Expect rpm to emulate OCM.
# 
# Here is an example on how to use this:
# 
# ```puppet
# ora_install::opatchupgrade{'112000_opatch_upgrade':
#   oracle_home               => '/oracle/product/11.2/db',
#   patch_file                => 'p6880880_112000_Linux-x86-64.zip',
#   csi_number                => '11111',
#   support_id                => 'john.doe@gmail.com',
#   csi_number                => undef,
#   support_id                => undef,
#   opversion                 => '11.2.0.3.6',
#   user                      => 'oracle',
#   group                     => 'dba',
#   download_dir              => '/install',
#   puppet_download_mnt_point => 'puppet:///modules/software',
# }
# ```
#
# @param [Optional[Integer]] csi_number
#    The Customer Service Identification number.
#    This is used to generate an OCMRF file.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [String[1]] opversion
#    The version of Opatch you want to install.
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] patch_file
#    The zip file containing the Opatch utility.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [Optional[String[1]]] support_id
#    Your support ID.
#    This is used to generate an OCMRF file.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
#--++--
define ora_install::opatchupgrade(
  Stdlib::Absolutepath                $oracle_home               = undef,
  String[1]                           $patch_file                = undef,
  Optional[Integer]                   $csi_number                = undef,
  Optional[String[1]]                 $support_id                = undef,
  String[1]                           $opversion                 = undef,
  String[1]                           $user                      = 'oracle',
  String[1]                           $group                     = 'dba',
  Stdlib::Absolutepath                $download_dir              = '/install',
  Optional[String[1]]                 $puppet_download_mnt_point = undef,
  Variant[Boolean,Enum['on_failure']] $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
){
  ora_install { $title: ensure => 'present'}

  $exec_path      = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'
  $patch_dir      = "${oracle_home}/OPatch"

  # if a mount was not specified then get the install media from the puppet master
  if ( $puppet_download_mnt_point ) {
    $mount_dir = $puppet_download_mnt_point
  } else {
    $mount_dir = 'puppet:///modules/ora_install'
  }

  # check the opatch version
  $installed_version = opatch_version($oracle_home)
  $version_parts = split($opversion,'\.')
  $major_opatch_version = $version_parts[0]
  #
  # major_opatch_version must be either `11` or `12` at this point in time
  #
  case $major_opatch_version {
    '11','12' : { # Do nothing, this is valid
    }
    default: {
      fail "Major opatch version number ${major_opatch_version} not recognised."
    }
  }

  unless ( $installed_version == $opversion ) {

    case $::kernel {
      'Linux', 'SunOS': {

        notify { "ora_install::opatchupgrade version ${installed_version} installed in ${oracle_home} - upgrade to ${opversion}": }

        archive { "${download_dir}/${title}_${patch_file}":
          ensure       => present,
          cleanup      => true,
          creates      => "${oracle_home}/OPatch/${opversion}",
          extract      => true,
          extract_path => $oracle_home,
          group        => $group,
          source       => "${mount_dir}/${patch_file}",
          user         => $user,
        }

        if ( ($major_opatch_version == '12' and versioncmp($opversion, '12.2.0.1.5') <= 0) or ($major_opatch_version == '11' and versioncmp($opversion, '11.2.0.3.14') <= 0) ) {
          #
          #
          # Older versions need the OCM file. Newer Opatch versions don't support this
          #
          if ( $csi_number and $support_id ) {
            exec { "exec emocmrsp ${title} ${opversion}":
              cwd       => $patch_dir,
              command   => "${patch_dir}/ocm/bin/emocmrsp -repeater NONE ${csi_number} ${support_id}",
              path      => $exec_path,
              user      => $user,
              group     => $group,
              logoutput => $logoutput,
              require   => Archive["${download_dir}/${title}_${patch_file}"],
            }
          } else {

            unless ( defined(Package['expect']) ) {
              package { 'expect':
                ensure  => present,
              }
            }

            file { "${download_dir}/opatch_upgrade_${title}_${opversion}.ksh":
              ensure  => present,
              content => template('ora_install/ocm.rsp.erb'),
              mode    => '0775',
              owner   => $user,
              group   => $group,
            }

            exec { "ksh ${download_dir}/opatch_upgrade_${title}_${opversion}.ksh":
              cwd       => $patch_dir,
              path      => $exec_path,
              user      => $user,
              group     => $group,
              logoutput => $logoutput,
              require   => [
                File["${download_dir}/opatch_upgrade_${title}_${opversion}.ksh"],
                Archive["${download_dir}/${title}_${patch_file}"],
                Package['expect'],
              ],
            }
          }
        }
        file { "${download_dir}/${title}_${patch_file}":
          ensure => absent,
        }
      }
      default: {
        fail('Unrecognized operating system')
      }
    }
  }
}
